@@ -1,4 +1,4 @@
-use crate::core::{Point, Signal};
+use crate::core::{ExternalSignal, Point, Signal};
 use crate::traits::{Source, Tracker};
 use alloc::vec;
 use alloc::vec::Vec;
@@ -11,26 +11,28 @@ use alloc::vec::Vec;
 ///
 /// const BUFFER_SIZE: usize = 16;
 /// let sample_rate = 4;
-/// let mut tracker = DynamicTracker::new();
-/// let mut primary = Primary::<BUFFER_SIZE>::new(&mut tracker, sample_rate);
+/// let mut primary = Primary::<BUFFER_SIZE>::new(sample_rate);
 /// let mut oscillator = Oscillator::new(&mut primary);
 ///
 /// oscillator.frequency = 1.0;
 /// oscillator.amplitude = 0.5;
 ///
-/// primary.add_monitor(&oscillator);
+/// primary.add_monitor(oscillator.output);
+/// primary.output_mono();
 ///
 /// assert_eq!(
 ///     primary.sample(vec![&mut oscillator]).unwrap(),
-///     vec![
-///         0.0, 0.0, 0.25, 0.25, 0.5, 0.5, -0.25, -0.25,
-///         0.0, 0.0, 0.25, 0.25, 0.5, 0.5, -0.25, -0.25,
-///         0.0, 0.0, 0.25, 0.25, 0.5, 0.5, -0.25, -0.25,
-///         0.0, 0.0, 0.25, 0.25, 0.5, 0.5, -0.25, -0.25,
+///     &[
+///         0.0, 0.25, 0.5, -0.25,
+///         0.0, 0.25, 0.5, -0.25,
+///         0.0, 0.25, 0.5, -0.25,
+///         0.0, 0.25, 0.5, -0.25,
 ///     ],
 /// );
 /// ```
 pub struct Oscillator {
+    /// main audio output
+    pub output: ExternalSignal,
     /// oscillator frequency per second
     pub frequency: f32,
     /// amplitude peak to peak centered around 0.0
@@ -38,7 +40,6 @@ pub struct Oscillator {
     /// for example an amplitude of `0.5` will generate a saw
     /// wave between `-0.5` and `0.5`
     pub amplitude: f32,
-    id: usize,
     value: Point,
     waveshape: Waveshape,
 }
@@ -54,7 +55,7 @@ impl Oscillator {
     /// frequency of `1.0` and an amplitute of `0.5`.
     pub fn new(tracker: &mut dyn Tracker) -> Self {
         Oscillator {
-            id: tracker.create_id(),
+            output: ExternalSignal::new(tracker.create_source_id(), 0),
             frequency: 1.0,
             amplitude: 0.5,
             value: 0.0,
@@ -65,28 +66,28 @@ impl Oscillator {
     /// Set the main output to triangle
     ///
     /// ```
-    /// use screech::core::{Primary, DynamicTracker};
+    /// use screech::core::Primary;
     /// use screech::basic::Oscillator;
     ///
     /// const BUFFER_SIZE: usize = 16;
     /// let sample_rate = 4;
-    /// let mut tracker = DynamicTracker::new();
-    /// let mut primary = Primary::<BUFFER_SIZE>::new(&mut tracker, sample_rate);
+    /// let mut primary = Primary::<BUFFER_SIZE>::new(sample_rate);
     /// let mut oscillator = Oscillator::new(&mut primary);
     ///
     /// oscillator.frequency = 0.5;
     /// oscillator.amplitude = 1.0;
     /// oscillator.output_triangle();
     ///
-    /// primary.add_monitor(&oscillator);
+    /// primary.add_monitor(oscillator.output);
+    /// primary.output_mono();
     ///
     /// assert_eq!(
     ///     primary.sample(vec![&mut oscillator]).unwrap(),
-    ///     vec![
-    ///         -1.0, -1.0, -0.5, -0.5, 0.0, 0.0, 0.5, 0.5,
-    ///         1.0, 1.0, 0.5, 0.5, 0.0, 0.0, -0.5, -0.5,
-    ///         -1.0, -1.0, -0.5, -0.5, 0.0, 0.0, 0.5, 0.5,
-    ///         1.0, 1.0, 0.5, 0.5, 0.0, 0.0, -0.5, -0.5,
+    ///     &[
+    ///         -1.0, -0.5, 0.0,  0.5,
+    ///          1.0,  0.5, 0.0, -0.5,
+    ///         -1.0, -0.5, 0.0,  0.5,
+    ///          1.0,  0.5, 0.0, -0.5,
     ///     ],
     /// );
     /// ```
@@ -99,13 +100,12 @@ impl Oscillator {
     /// with a duty cycle between `0.0` (0%) and `1.0` (100%).
     ///
     /// ```
-    /// use screech::core::{Primary, DynamicTracker};
+    /// use screech::core::Primary;
     /// use screech::basic::Oscillator;
     ///
     /// const BUFFER_SIZE: usize = 4;
     /// let sample_rate = 4;
-    /// let mut tracker = DynamicTracker::new();
-    /// let mut primary = Primary::<BUFFER_SIZE>::new(&mut tracker, sample_rate);
+    /// let mut primary = Primary::<BUFFER_SIZE>::new(sample_rate);
     /// let mut oscillator = Oscillator::new(&mut primary);
     ///
     /// oscillator.frequency = 1.0;
@@ -113,11 +113,12 @@ impl Oscillator {
     /// // 25% duty cycle
     /// oscillator.output_square(0.25);
     ///
-    /// primary.add_monitor(&oscillator);
+    /// primary.add_monitor(oscillator.output);
+    /// primary.output_mono();
     ///
     /// assert_eq!(
-    ///          primary.sample(vec![&mut oscillator]).unwrap(),
-    ///         vec![-1.0, -1.0, -1.0, -1.0, -1.0, -1.0, 1.0, 1.0],
+    ///         primary.sample(vec![&mut oscillator]).unwrap(),
+    ///         &[-1.0, -1.0, -1.0, 1.0],
     /// );
     /// ```
     pub fn output_square(&mut self, duty_cycle: f32) -> &mut Self {
@@ -128,26 +129,26 @@ impl Oscillator {
     /// Set the main output to saw
     ///
     /// ```
-    /// use screech::core::{Primary, DynamicTracker};
+    /// use screech::core::Primary;
     /// use screech::basic::Oscillator;
     ///
     /// const BUFFER_SIZE: usize = 8;
     /// let sample_rate = 4;
-    /// let mut tracker = DynamicTracker::new();
-    /// let mut primary = Primary::<BUFFER_SIZE>::new(&mut tracker, sample_rate);
+    /// let mut primary = Primary::<BUFFER_SIZE>::new(sample_rate);
     /// let mut oscillator = Oscillator::new(&mut primary);
     ///
     /// oscillator.frequency = 0.5;
     /// oscillator.amplitude = 1.0;
     /// oscillator.output_saw();
     ///
-    /// primary.add_monitor(&oscillator);
+    /// primary.add_monitor(oscillator.output);
+    /// primary.output_mono();
     ///
     /// assert_eq!(
     ///         primary.sample(vec![&mut oscillator]).unwrap(),
-    ///         vec![
-    ///              0.0, 0.0,  0.25,  0.25,  0.5,  0.5,  0.75,  0.75,
-    ///              1.0, 1.0, -0.75, -0.75, -0.5, -0.5, -0.25, -0.25,
+    ///         &[
+    ///              0.0,  0.25,  0.5,  0.75,
+    ///              1.0, -0.75, -0.5, -0.25,
     ///         ],
     /// );
     /// ```
@@ -155,9 +156,10 @@ impl Oscillator {
         self.waveshape = Waveshape::Saw;
         self
     }
+}
 
-    /// Render the next real time signal
-    pub fn step(&mut self, sample_rate: usize) -> Signal {
+impl Source for Oscillator {
+    fn sample(&mut self, sources: &mut dyn Tracker, sample_rate: usize) {
         // peak to peak conversion of amplitude
         let peak_to_peak = self.amplitude * 2.0;
 
@@ -191,17 +193,11 @@ impl Oscillator {
             self.value -= peak_to_peak;
         }
 
-        Signal::point(point)
-    }
-}
-
-impl Source for Oscillator {
-    fn sample(&mut self, sources: &mut dyn Tracker, sample_rate: usize) {
-        sources.set_signal(self.id, self.step(sample_rate));
+        sources.set_signal(&self.output, Signal::point(point));
     }
 
-    fn get_id(&self) -> usize {
-        self.id
+    fn get_source_id(&self) -> &usize {
+        self.output.get_source_id()
     }
 
     fn get_sources(&self) -> Vec<usize> {
@@ -212,46 +208,42 @@ impl Source for Oscillator {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::core::{DynamicTracker, Primary};
+    use crate::core::Primary;
 
     #[test]
     fn test_basic_repetition() {
-        let mut tracker = DynamicTracker::new();
-        let mut primary = Primary::<{ 4 * 4 }>::new(&mut tracker, 4);
+        let mut primary = Primary::<{ 4 * 4 }>::new(4);
         let mut oscillator = Oscillator::new(&mut primary);
 
         oscillator.frequency = 1.0;
         oscillator.amplitude = 0.25;
 
-        primary.add_monitor(&oscillator);
+        primary.add_monitor(oscillator.output);
+        primary.output_mono();
 
         assert_eq!(
             primary.sample(vec![&mut oscillator]).unwrap(),
-            vec![
-                0.0, 0.0, 0.125, 0.125, 0.25, 0.25, -0.125, -0.125, 0.0, 0.0, 0.125, 0.125, 0.25,
-                0.25, -0.125, -0.125, 0.0, 0.0, 0.125, 0.125, 0.25, 0.25, -0.125, -0.125, 0.0, 0.0,
-                0.125, 0.125, 0.25, 0.25, -0.125, -0.125,
+            &[
+                0.0, 0.125, 0.25, -0.125, 0.0, 0.125, 0.25, -0.125, 0.0, 0.125, 0.25, -0.125, 0.0,
+                0.125, 0.25, -0.125,
             ],
         );
     }
 
     #[test]
     fn test_repeat_every_other_second() {
-        let mut tracker = DynamicTracker::new();
-        let mut primary = Primary::<{ 4 * 3 }>::new(&mut tracker, 4);
+        let mut primary = Primary::<{ 4 * 3 }>::new(4);
         let mut oscillator = Oscillator::new(&mut primary);
 
         oscillator.frequency = 1.5;
         oscillator.amplitude = 0.5;
 
-        primary.add_monitor(&oscillator);
+        primary.add_monitor(oscillator.output);
+        primary.output_mono();
 
         assert_eq!(
             primary.sample(vec![&mut oscillator]).unwrap(),
-            vec![
-                0.0, 0.0, 0.375, 0.375, -0.25, -0.25, 0.125, 0.125, 0.5, 0.5, -0.125, -0.125, 0.25,
-                0.25, -0.375, -0.375, 0.0, 0.0, 0.375, 0.375, -0.25, -0.25, 0.125, 0.125
-            ],
+            &[0.0, 0.375, -0.25, 0.125, 0.5, -0.125, 0.25, -0.375, 0.0, 0.375, -0.25, 0.125],
         );
     }
 }
