@@ -1,3 +1,4 @@
+mod external_signal;
 mod graph;
 mod primary;
 mod signal;
@@ -7,6 +8,7 @@ mod tracker;
 /// core data type representing audio data and utilities
 pub mod point;
 
+pub use external_signal::ExternalSignal;
 pub use point::Point;
 pub use primary::Primary;
 pub use signal::Signal;
