@@ -1,6 +1,7 @@
 use crate::core::graph::{topological_sort, Error as GraphError};
-use crate::core::{Point, Signal};
+use crate::core::{DynamicTracker, ExternalSignal, Point, Signal};
 use crate::traits::{Source, Tracker};
+use alloc::boxed::Box;
 use alloc::vec;
 use alloc::vec::Vec;
 use rustc_hash::FxHashMap;
@@ -13,41 +14,41 @@ use rustc_hash::FxHashMap;
 /// use screech::core::{Primary, Stream, DynamicTracker};
 /// use screech::basic::{Clip, Track};
 ///
-/// const BUFFER_SIZE: usize = 2;
+/// const BUFFER_SIZE: usize = 4;
 /// let sample_rate = 48_000;
 ///
-/// let mut tracker = DynamicTracker::new();
-/// let mut primary = Primary::<BUFFER_SIZE>::new(&mut tracker, sample_rate);
+/// let mut primary = Primary::<BUFFER_SIZE>::new(sample_rate);
+///
 /// let mut clip_a = Clip::new(&mut primary, Stream::from_points(vec![0.1, 0.2, 0.2, 0.1]));
 /// let mut clip_b = Clip::new(&mut primary, Stream::from_points(vec![0.0, 0.0, 0.1, 0.3]));
 /// let mut track = Track::new(&mut primary);
 ///
-/// track.add_input(&clip_a);
-/// track.add_input(&clip_b);
-/// primary.add_monitor(&track);
+/// track.add_input(clip_a.output);
+/// track.add_input(clip_b.output);
+/// primary.add_monitor(track.output);
 ///
 /// assert_eq!(
 ///     primary.sample(vec![&mut clip_a, &mut clip_b, &mut track]).unwrap(),
-///     vec![0.1, 0.1, 0.2, 0.2],
+///     &[0.1, 0.1, 0.2, 0.2],
 /// );
 ///
 /// assert_eq!(
 ///     primary.sample(vec![&mut clip_a, &mut clip_b, &mut track]).unwrap(),
-///     vec![0.3, 0.3, 0.4, 0.4],
+///     &[0.3, 0.3, 0.4, 0.4],
 /// );
 ///
 /// assert_eq!(
 ///     primary.sample(vec![&mut clip_a, &mut clip_b, &mut track]).unwrap(),
-///     vec![0.0, 0.0, 0.0, 0.0],
+///     &[0.0, 0.0, 0.0, 0.0],
 /// );
 /// ```
-pub struct Primary<'a, const BUFFER_SIZE: usize> {
-    buffer: [Signal; BUFFER_SIZE],
-    sample_rate: usize,
-    monitored_sources: Vec<usize>,
+pub struct Primary<const BUFFER_SIZE: usize> {
+    buffer: [f32; BUFFER_SIZE],
+    /// sample rate field used for sampling
+    pub sample_rate: usize,
+    monitored_sources: Vec<ExternalSignal>,
     output_mode: OutputMode,
-    // tracker: BasicTracker<SOURCES_SIZE>,
-    tracker: &'a mut dyn Tracker,
+    tracker: Box<dyn Tracker>,
 }
 
 enum OutputMode {
@@ -70,29 +71,39 @@ pub enum Error {
     MissingDependency,
 }
 
-impl<'a, const BUFFER_SIZE: usize> Primary<'a, BUFFER_SIZE> {
-    /// Create new Primary "channel"
-    pub fn new(tracker: &'a mut dyn Tracker, sample_rate: usize) -> Self {
+impl<const BUFFER_SIZE: usize> Primary<BUFFER_SIZE> {
+    /// Create new Primary with a default tracker
+    pub fn new(sample_rate: usize) -> Self {
+        Primary::with_tracker(Box::new(DynamicTracker::new()), sample_rate)
+    }
+
+    /// Create a new Primary with a supplied tracker
+    ///
+    /// ```
+    /// use screech::core::{Primary, BasicTracker};
+    ///
+    /// let tracker = BasicTracker::<256, 8>::new();
+    /// let primary = Primary::<128>::with_tracker(Box::new(tracker), 48_000);
+    /// ```
+    pub fn with_tracker(tracker: Box<dyn Tracker>, sample_rate: usize) -> Self {
         Primary {
-            buffer: [Signal::silence(); BUFFER_SIZE],
+            buffer: [0.0; BUFFER_SIZE],
             sample_rate,
             monitored_sources: vec![],
             output_mode: OutputMode::Stereo,
-            // tracker: BasicTracker::<SOURCES_SIZE>::new(),
             tracker,
         }
     }
 
     /// add source to the final output
-    pub fn add_monitor(&mut self, source: &dyn Source) -> &mut Self {
-        self.monitored_sources.push(source.get_id());
+    pub fn add_monitor(&mut self, source: ExternalSignal) -> &mut Self {
+        self.monitored_sources.push(source);
         self
     }
 
     /// remove source from final output
-    pub fn remove_monitor(&mut self, source: &dyn Source) -> &mut Self {
-        let a = source.get_id();
-        self.monitored_sources.retain(|&b| a != b);
+    pub fn remove_monitor(&mut self, source: &ExternalSignal) -> &mut Self {
+        self.monitored_sources.retain(|b| source != b);
         self
     }
 
@@ -109,13 +120,16 @@ impl<'a, const BUFFER_SIZE: usize> Primary<'a, BUFFER_SIZE> {
     }
 
     /// Sample multiple sources based on their dependencies into a single output vec
-    pub fn sample(&mut self, unmapped_sources: Vec<&mut dyn Source>) -> Result<Vec<Point>, Error> {
+    pub fn sample(
+        &mut self,
+        unmapped_sources: Vec<&mut dyn Source>,
+    ) -> Result<&[Point; BUFFER_SIZE], Error> {
         let mut sources = FxHashMap::<usize, &mut dyn Source>::default();
         let mut graph = FxHashMap::<usize, Vec<usize>>::default();
 
         for source in unmapped_sources {
-            graph.insert(source.get_id(), source.get_sources());
-            sources.insert(source.get_id(), source);
+            graph.insert(*source.get_source_id(), source.get_sources());
+            sources.insert(*source.get_source_id(), source);
         }
 
         let sorted = topological_sort(graph).map_err(|e| match e {
@@ -130,58 +144,56 @@ impl<'a, const BUFFER_SIZE: usize> Primary<'a, BUFFER_SIZE> {
         }
 
         let sample_rate = self.sample_rate;
+        let loop_size = match self.output_mode {
+            OutputMode::Mono => BUFFER_SIZE,
+            OutputMode::Stereo => BUFFER_SIZE / 2,
+        };
 
-        for i in 0..BUFFER_SIZE {
+        for i in 0..loop_size {
             for source in sorted_sources.iter_mut() {
                 source.sample(self, sample_rate);
             }
 
-            let signals: Vec<&Signal> = self
-                .monitored_sources
-                .iter()
-                .filter_map(|&k| self.get_signal(k))
-                .collect();
+            let mut signals = Vec::with_capacity(self.monitored_sources.len());
+            for external_signal in self.monitored_sources.iter() {
+                signals.push(
+                    self.get_signal(&external_signal)
+                        .ok_or(Error::MissingMonitor)?,
+                );
+            }
 
-            self.buffer[i] = Signal::mix(&signals);
-        }
-
-        let mut output = Vec::with_capacity(BUFFER_SIZE * 2);
-
-        for signal in self.buffer {
             match self.output_mode {
                 OutputMode::Mono => {
-                    let point = signal.sum_points();
-                    output.push(point);
+                    self.buffer[i] = Signal::mix(&signals).sum_points();
                 }
                 OutputMode::Stereo => {
-                    let left = signal.get_point();
-                    let right = signal.get_right_point().unwrap_or(signal.get_point());
-
-                    output.push(*left);
-                    output.push(*right);
+                    let signal = Signal::mix(&signals);
+                    self.buffer[i * 2] = *signal.get_point();
+                    self.buffer[i * 2 + 1] =
+                        *signal.get_right_point().unwrap_or(signal.get_point());
                 }
-            }
+            };
         }
 
-        Ok(output)
+        Ok(&self.buffer)
     }
 }
 
-impl<'a, const A: usize> Tracker for Primary<'a, A> {
-    fn create_id(&mut self) -> usize {
-        self.tracker.create_id()
+impl<const A: usize> Tracker for Primary<A> {
+    fn create_source_id(&mut self) -> usize {
+        self.tracker.create_source_id()
     }
 
-    fn clear_id(&mut self, id: usize) {
-        self.tracker.clear_id(id)
+    fn clear_source(&mut self, external_signal: usize) {
+        self.tracker.clear_source(external_signal)
     }
 
-    fn get_signal(&self, id: usize) -> Option<&Signal> {
-        self.tracker.get_signal(id)
+    fn get_signal(&self, external_signal: &ExternalSignal) -> Option<&Signal> {
+        self.tracker.get_signal(external_signal)
     }
 
-    fn set_signal(&mut self, id: usize, signal: Signal) {
-        self.tracker.set_signal(id, signal);
+    fn set_signal(&mut self, external_signal: &ExternalSignal, signal: Signal) {
+        self.tracker.set_signal(external_signal, signal);
     }
 }
 
@@ -189,13 +201,12 @@ impl<'a, const A: usize> Tracker for Primary<'a, A> {
 mod tests {
     use super::*;
     use crate::basic::{Clip, Track};
-    use crate::core::{DynamicTracker, Stream};
+    use crate::core::Stream;
     use crate::traits::FromPoints;
 
     #[test]
     fn test_complex_dependencies() {
-        let mut tracker = DynamicTracker::new();
-        let mut primary = Primary::<5>::new(&mut tracker, 48_000);
+        let mut primary = Primary::<5>::new(48_000);
 
         let mut clip_a = Clip::new(&mut primary, Stream::from_points(vec![0.1]));
         let mut clip_b = Clip::new(&mut primary, Stream::from_points(vec![0.0, 0.2]));
@@ -211,18 +222,18 @@ mod tests {
         let mut track_c = Track::new(&mut primary);
         let mut track_d = Track::new(&mut primary);
 
-        track_a.add_input(&clip_a).add_input(&clip_b);
+        track_a.add_input(clip_a.output).add_input(clip_b.output);
 
-        track_b.add_input(&track_a).add_input(&clip_c);
+        track_b.add_input(track_a.output).add_input(clip_c.output);
 
         track_c
-            .add_input(&track_b)
-            .add_input(&clip_d)
-            .add_input(&clip_e);
+            .add_input(track_b.output)
+            .add_input(clip_d.output)
+            .add_input(clip_e.output);
 
-        track_d.add_input(&track_c);
+        track_d.add_input(track_c.output);
 
-        primary.add_monitor(&track_d);
+        primary.add_monitor(track_d.output);
         primary.output_mono();
 
         assert_eq!(
@@ -237,25 +248,24 @@ mod tests {
                 &mut track_c,
                 &mut track_d,
             ]),
-            Ok(vec![0.1, 0.2, 0.3, 0.4, 0.5]),
+            Ok(&[0.1, 0.2, 0.3, 0.4, 0.5]),
         );
     }
 
     #[test]
     fn test_dependency_failure() {
-        let mut tracker = DynamicTracker::new();
-        let mut primary = Primary::<2>::new(&mut tracker, 48_000);
+        let mut primary = Primary::<2>::new(48_000);
         let mut clip_a = Clip::new(&mut primary, Stream::from_points(vec![0.1, 0.2, 0.2, 0.1]));
         let clip_b = Clip::new(&mut primary, Stream::from_points(vec![0.0, 0.0, 0.1, 0.3]));
         let clip_c = Clip::new(&mut primary, Stream::from_points(vec![0.0, 0.0, 0.1, 0.3]));
         let mut track = Track::new(&mut primary);
 
         track
-            .add_input(&clip_a)
-            .add_input(&clip_b)
-            .add_input(&clip_c);
+            .add_input(clip_a.output)
+            .add_input(clip_b.output)
+            .add_input(clip_c.output);
 
-        primary.add_monitor(&track);
+        primary.add_monitor(track.output);
 
         assert_eq!(
             primary.sample(vec![&mut clip_a, &mut track]),
@@ -265,19 +275,30 @@ mod tests {
 
     #[test]
     fn test_circular_dependency_failure() {
-        let mut tracker = DynamicTracker::new();
-        let mut primary = Primary::<2>::new(&mut tracker, 48_000);
+        let mut primary = Primary::<2>::new(48_000);
         let mut track_a = Track::new(&mut primary);
         let mut track_b = Track::new(&mut primary);
 
-        track_a.add_input(&track_b);
-        track_b.add_input(&track_a);
+        track_a.add_input(track_b.output);
+        track_b.add_input(track_a.output);
 
-        primary.add_monitor(&track_a).add_monitor(&track_b);
+        primary
+            .add_monitor(track_a.output)
+            .add_monitor(track_b.output);
 
         assert_eq!(
             primary.sample(vec![&mut track_a, &mut track_b]),
             Err(Error::CyclicDependencies),
         );
     }
+
+    #[test]
+    fn test_circular_missing_monitor_failure() {
+        let mut primary = Primary::<2>::new(48_000);
+        let track = Track::new(&mut primary);
+
+        primary.add_monitor(track.output);
+
+        assert_eq!(primary.sample(vec![]), Err(Error::MissingMonitor),);
+    }
 }
